/*
 * @file ISR.c
 * @brief Definicao de prototipos relativos a rotinas de servico do projeto relogio_digital
 * @date 08/02/2024
 * @author Wu Shin-Ting
 */
#include "derivative.h"
#include "ISR.h"
#include "RTC.h"
#include "PIT.h"
#include "util.h"

static tipo_estado estado;
static uint32_t counter;
static uint32_t hor[4];

void ISR_carregaHorario() {
	uint32_t segundos;

	RTClpo_getTime (&segundos);
	ConvertSectoDay(segundos, &hor[3], &hor[0], &hor[1], &hor[2]);
}

void ISR_leHorario (uint32_t *dias, uint32_t *horas, uint32_t *minutos, uint32_t *segundos)
{
		*dias = hor[3];
		*horas = hor[0];
		*minutos = hor[1];
		*segundos = hor[2];

		return;
}

tipo_estado ISR_LeEstado ()
{	
	return estado;
}

void ISR_escreveEstado (tipo_estado s)
{	
	estado = s;
}

void RTC_Seconds_IRQHandler() {
	GPIOE_PTOR |= GPIO_PTOR_PTTO (GPIO_PIN(22));
}

void PIT_IRQHandler () {
	uint32_t segundos;
	
	counter++;
	
	//para analise do periodo configurado para PIT
	GPIOE_PTOR |= GPIO_PTOR_PTTO (GPIO_PIN(21));
	
	if (counter >= POSTSCALER) {
		//para analise do periodo * postscaler configurado para PIT
		GPIOE_PTOR |= GPIO_PTOR_PTTO (GPIO_PIN(20));

		ConvertDaytoSec (hor[3], hor[0], hor[1], hor[2], &segundos);
		RTClpo_setTime (segundos);
				
		PIT_desativaTimer0 ();
		estado = NORMAL;		
	}
	
	//w1c
	PIT_TFLG0 |= PIT_TFLG_TIF_MASK;

	return;
}

void PORTA_IRQHandler () {
	//Reativa PIT 
	counter = 0;
	PIT_resetaCVAL0();
	
	GPIOE_PTOR |= GPIO_PTOR_PTTO (GPIO_PIN(20));

	switch (estado) {
	case NORMAL:
		ISR_carregaHorario();

		if (PORTA_ISFR & PORT_ISFR_ISF(GPIO_PIN(5))) {
			estado = INCREMENTE_MINUTO;
		} 
		break;
	case INCREMENTE_HORA:
	case INCREMENTE_HORA_ESPERA:
		break;
	case INCREMENTE_MINUTO:
	case INCREMENTE_MINUTO_ESPERA:
		if (PORTA_ISFR & PORT_ISFR_ISF(GPIO_PIN(5))) {
			hor[1]++;
			if (hor[1] >= 60) hor[1] = 0;
			estado = INCREMENTE_MINUTO;			
		} 
		break;
	case INCREMENTE_SEGUNDO:
	case INCREMENTE_SEGUNDO_ESPERA:
		break;
	}
	
	//Limpar o estado de solicitacao
	PORTA_ISFR |= PORT_ISFR_ISF(GPIO_PIN(5)); 
}
